@@ -40,11 +40,11 @@ bool CDefaultFilter::DecorateView(CDasherView *pView) {
   return bDidSomething;
 }
 
-bool CDefaultFilter::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel) {
+bool CDefaultFilter::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted) {
   myint iDasherX;
   myint iDasherY;
 
-  m_pDasherView->TapOnDisplay(0, 0, Time, iDasherX, iDasherY);
+  m_pDasherView->GetCoordinates(Time, iDasherX, iDasherY);
 
   ApplyAutoCalibration(iDasherX, iDasherY, true);
   ApplyTransform(iDasherX, iDasherY);
@@ -61,7 +61,7 @@ bool CDefaultFilter::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m
   }
 
   bool bDidSomething;
-  bDidSomething = m_pDasherModel->Tap_on_display(iDasherX,iDasherY, Time, 0, 0);
+  bDidSomething = m_pDasherModel->UpdatePosition(iDasherX,iDasherY, Time, pAdded, pNumDeleted);
 
   m_pAutoSpeedControl->SpeedControl(iDasherX, iDasherY, m_pDasherModel->Framerate(), m_pDasherView);
 
@@ -71,7 +71,7 @@ bool CDefaultFilter::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m
   return bDidSomething;
 }
 
-void CDefaultFilter::KeyDown(int iTime, int iId, CDasherModel *pModel) {
+void CDefaultFilter::KeyDown(int iTime, int iId, CDasherModel *pModel, CUserLogBase *pUserLog) {
 
   switch(iId) {
   case 0: // Start on space
@@ -126,7 +126,8 @@ void CDefaultFilter::DrawMouse(CDasherView *pView) {
 
   myint *pCoordinates(new myint[iCoordinateCount]);
 
-  int iType(pView->GetCoordinates(iCoordinateCount, pCoordinates));
+  // TODO: redundant?
+  int iType(pView->GetInputCoordinates(iCoordinateCount, pCoordinates));
 
   screenint mousex;
   screenint mousey;
@@ -144,18 +145,12 @@ void CDefaultFilter::DrawMouse(CDasherView *pView) {
   myint iDasherX;
   myint iDasherY;
 
-  pView->Input2Dasher(mousex, mousey, iDasherX, iDasherY, iType, 0);
+  pView->Screen2Dasher(mousex, mousey, iDasherX, iDasherY, false, true);
 
   ApplyAutoCalibration(iDasherX, iDasherY, false);
   ApplyTransform(iDasherX, iDasherY);
 
-  if(GetBoolParameter(BP_COLOUR_MODE) == true) {
-    pView->DasherDrawCentredRectangle(iDasherX, iDasherY, 5, 2, Opts::ColorSchemes(Objects), false);
-  }
-  else {
-    pView->DasherDrawCentredRectangle(iDasherX, iDasherY, 5, 1, Opts::ColorSchemes(Objects), false);
-  }
-
+  pView->DasherDrawCentredRectangle(iDasherX, iDasherY, 5, 2, Opts::ColorSchemes(Objects), false);
 }
 
 void CDefaultFilter::DrawMouseLine(CDasherView *pView) {
@@ -164,7 +159,8 @@ void CDefaultFilter::DrawMouseLine(CDasherView *pView) {
 
   myint *pCoordinates(new myint[iCoordinateCount]);
 
-  int iType(pView->GetCoordinates(iCoordinateCount, pCoordinates));
+  // TODO: Redundant, plus code duplication here
+  int iType(pView->GetInputCoordinates(iCoordinateCount, pCoordinates));
 
   screenint mousex;
   screenint mousey;
@@ -191,21 +187,15 @@ void CDefaultFilter::DrawMouseLine(CDasherView *pView) {
   // End of line is the mouse cursor location - note that we should
   // probably be using a chached value rather than computing this
   // separately to TapOn
-
  
-  pView->Input2Dasher(mousex, mousey, x[1], y[1], iType, 0);
+  pView->Screen2Dasher(mousex, mousey, x[1], y[1], false, true);
 
   ApplyAutoCalibration(x[1], y[1], false);
   ApplyTransform(x[1], y[1]);
 
   // Actually plot the line
 
-  if(GetBoolParameter(BP_COLOUR_MODE)) {
-    pView->DasherPolyline(x, y, 2, GetLongParameter(LP_LINE_WIDTH), 1);
-  }
-  else {
-    pView->DasherPolyline(x, y, 2, GetLongParameter(LP_LINE_WIDTH), -1);
-  }
+  pView->DasherPolyline(x, y, 2, GetLongParameter(LP_LINE_WIDTH), 1);
 }
 
 void CDefaultFilter::ApplyTransform(myint &iDasherX, myint &iDasherY) {
