@@ -7,8 +7,14 @@ class COneDimensionalFilter : public CDefaultFilter {
  public:
   COneDimensionalFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface, CDasherModel *m_pDasherModel);
 
+  bool GetSettings(SModuleSettings **pSettings, int *iCount);
+  virtual void KeyDown(int iTime, int iId, CDasherModel *pModel, CUserLogBase *pUserLog);
+
  private:
    virtual void ApplyTransform(myint &iDasherX, myint &iDasherY);
+
+   int iLastY;
+   int iOffset;
 };
 
 #endif
