@@ -5,9 +5,7 @@
 #ifndef WITH_MAEMO
 #include "dasher_action_copy.h"
 #endif
-#ifdef GNOME_A11Y
 #include "dasher_action_keyboard.h"
-#endif 
 #ifdef WITH_MAEMO
 #include "dasher_action_keyboard_maemo.h"
 #else
@@ -24,6 +22,7 @@
 #include "game_mode_helper.h"
 #include "FontDialogues.h"
 #include "GtkDasherControl.h"
+#include "Menu.h"
 #include "Preferences.h"
 
 // TODO: Maybe reimplement something along the lines of the following, which used to be in edit.cc
@@ -150,13 +149,16 @@ static void dasher_editor_destroy(GObject *pObject) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(((DasherEditor *)pObject)->private_data);
 
   EditorAction *pCurrentAction = pPrivate->pActionRing;
-  bool bStarted = false;
-  
-  while(!bStarted || (pCurrentAction != pPrivate->pActionRing)) {
-    bStarted = true;
-    dasher_action_deactivate(pCurrentAction->pAction);
-    g_object_unref(G_OBJECT(pCurrentAction->pAction));
-    pCurrentAction = pCurrentAction->pNext;
+
+  if(pCurrentAction) {
+    bool bStarted = false;
+    
+    while(!bStarted || (pCurrentAction != pPrivate->pActionRing)) {
+      bStarted = true;
+      dasher_action_deactivate(pCurrentAction->pAction);
+      g_object_unref(G_OBJECT(pCurrentAction->pAction));
+      pCurrentAction = pCurrentAction->pNext;
+    }
   }
 
   if(pPrivate->pBufferSet)
@@ -168,29 +170,15 @@ static void dasher_editor_destroy(GObject *pObject) {
 DasherEditor *dasher_editor_new(int argc, char **argv) {
   DasherEditor *pDasherControl;
   pDasherControl = (DasherEditor *)(g_object_new(dasher_editor_get_type(), NULL));
-  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pDasherControl->private_data);
 
-  g_pDasherAppSettings = dasher_app_settings_new(argc, argv);
+  return pDasherControl;
+}
 
-  g_pDasherMain = dasher_main_new();
-  //  g_signal_connect(G_OBJECT(g_pDasherMain), "realized", G_CALLBACK(main_window_realized), pDasherControl);
+void dasher_editor_initialise(DasherEditor *pSelf) {
+  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
 
   GladeXML *pGladeXML = dasher_main_get_glade(g_pDasherMain);
-  pDasherWidget = glade_xml_get_widget(pGladeXML, "DasherControl");
   
-  dasher_main_set_app_settings(g_pDasherMain, g_pDasherAppSettings);
-
-  // TODO: Make lock diaogue a full method
-#ifndef WITH_MAEMO
-  dasher_lock_dialogue_new(pGladeXML, GTK_WINDOW(dasher_main_get_window(g_pDasherMain)));
-#else
-  dasher_lock_dialogue_new(pGladeXML, 0);
-#endif
-  // TODO: Bring into object framework
-  InitialiseFontDialogues(pGladeXML);
-  
-  g_pPreferencesDialogue = dasher_preferences_dialogue_new(pGladeXML, pDasherControl);
-
   GtkTextView *pTextView = GTK_TEXT_VIEW(glade_xml_get_widget(pGladeXML, "the_text_view"));
   GtkVBox *pActionPane = GTK_VBOX(glade_xml_get_widget(pGladeXML, "vbox39"));
 
@@ -203,11 +191,9 @@ DasherEditor *dasher_editor_new(int argc, char **argv) {
   pPrivate->iNextActionID = 0;
   pPrivate->pGameModeHelper = 0;
 
-  dasher_editor_create_buffer(pDasherControl);
-  dasher_editor_setup_actions(pDasherControl);
-  dasher_preferences_dialogue_populate_actions(g_pPreferencesDialogue);
- 
-  return pDasherControl;
+  dasher_editor_create_buffer(pSelf);
+  dasher_editor_setup_actions(pSelf);
+  //
 }
 
 IDasherBufferSet *dasher_editor_get_buffer_set(DasherEditor *pSelf) {
@@ -280,13 +266,16 @@ void dasher_editor_handle_stop(DasherEditor *pSelf) {
   
   // See if anything is set to auto:
   EditorAction *pCurrentAction = pPrivate->pActionRing;
-  bool bStarted = false;
-  
-  while(!bStarted || (pCurrentAction != pPrivate->pActionRing)) {
-    bStarted = true;
-    if(pCurrentAction->bAuto)
-      dasher_action_execute(pCurrentAction->pAction, dasher_editor_get_all_text(pSelf)); 
-    pCurrentAction = pCurrentAction->pNext;
+
+  if(pCurrentAction) {
+    bool bStarted = false;
+    
+    while(!bStarted || (pCurrentAction != pPrivate->pActionRing)) {
+      bStarted = true;
+      if(pCurrentAction->bAuto)
+	dasher_action_execute(pCurrentAction->pAction, dasher_editor_get_all_text(pSelf)); 
+      pCurrentAction = pCurrentAction->pNext;
+    }
   }
 }
 
@@ -345,22 +334,24 @@ void dasher_editor_handle_control(DasherEditor *pSelf, int iNodeID) {
     {Dasher::CControlManager::CTL_DELETE_BACKWARD_FILE, EDIT_BACKWARDS, EDIT_FILE, true}
   };    
 
-  for(int i(0); i < sizeof(sMap)/sizeof(struct SControlMap); ++i) {
-    if(sMap[i].iEvent == iNodeID) {
-      if(sMap[i].bDelete) 
-	idasher_buffer_set_edit_delete(pPrivate->pBufferSet, sMap[i].iDir, sMap[i].iDist);
-      else
-	idasher_buffer_set_edit_move(pPrivate->pBufferSet, sMap[i].iDir, sMap[i].iDist);	
+  if(pPrivate->pBufferSet) {
+    for(unsigned int i(0); i < sizeof(sMap)/sizeof(struct SControlMap); ++i) {
+      if(sMap[i].iEvent == iNodeID) {
+	if(sMap[i].bDelete) 
+	  idasher_buffer_set_edit_delete(pPrivate->pBufferSet, sMap[i].iDir, sMap[i].iDist);
+	else
+	  idasher_buffer_set_edit_move(pPrivate->pBufferSet, sMap[i].iDir, sMap[i].iDist);	
+      }
     }
   }
 }
 
 void dasher_editor_handle_parameter_change(DasherEditor *pSelf, int iParameter) {
-  switch(iParameter) {
-  case APP_LP_STYLE:
-    dasher_editor_create_buffer(pSelf);
-    break;
-  }
+//   switch(iParameter) {
+//   case APP_LP_STYLE:
+//     dasher_editor_create_buffer(pSelf);
+//     break;
+//   }
 
   dasher_preferences_dialogue_handle_parameter_change(g_pPreferencesDialogue, iParameter);
   dasher_main_handle_parameter_change(g_pDasherMain, iParameter);
@@ -382,7 +373,7 @@ void dasher_editor_add_action(DasherEditor *pSelf, DasherAction *pNewAction) {
   strncpy(szRegistryName, "Action_", 256);
   strncat(szRegistryName, dasher_action_get_name(pNewEditorAction->pAction), 255 - strlen(szRegistryName));
 
-  for(int i(0); i < strlen(szRegistryName); ++i)
+  for(unsigned int i(0); i < strlen(szRegistryName); ++i)
     if(szRegistryName[i] == ' ')
       szRegistryName[i] = '_';
 
@@ -415,6 +406,9 @@ void dasher_editor_add_action(DasherEditor *pSelf, DasherAction *pNewAction) {
   }
 
   pPrivate->pActionRing = pNewEditorAction;
+  
+  if(iState & ACTION_STATE_SHOW)
+  	gtk_dasher_control_add_action_button(GTK_DASHER_CONTROL(pDasherWidget), dasher_action_get_name(pNewEditorAction->pAction));
 }
 
 void dasher_editor_setup_actions(DasherEditor *pSelf) {
@@ -427,9 +421,7 @@ void dasher_editor_setup_actions(DasherEditor *pSelf) {
   dasher_editor_add_action(pSelf, DASHER_ACTION(dasher_action_speech_new()));
 #endif
 
-#ifdef GNOME_A11Y
   dasher_editor_add_action(pSelf, DASHER_ACTION(dasher_action_keyboard_new(pPrivate->pExternalBuffer)));
-#endif
 
 #ifdef WITH_MAEMO
   dasher_editor_add_action(pSelf, DASHER_ACTION(dasher_action_keyboard_maemo_new()));
@@ -497,7 +489,7 @@ void dasher_editor_setup_actions(DasherEditor *pSelf) {
   }
 
 #ifndef WITH_MAEMOFULLSCREEN
-  dasher_editor_rebuild_action_pane(pSelf);
+  //  dasher_editor_rebuild_action_pane(pSelf);
 #endif
 }
 
@@ -662,7 +654,7 @@ void dasher_editor_action_save_state(DasherEditor *pSelf, EditorAction *pAction)
   strncpy(szRegistryName, "Action_", 256);
   strncat(szRegistryName, dasher_action_get_name(pAction->pAction), 255 - strlen(szRegistryName));
 
-  for(int i(0); i < strlen(szRegistryName); ++i)
+  for(unsigned int i(0); i < strlen(szRegistryName); ++i)
     if(szRegistryName[i] == ' ')
       szRegistryName[i] = '_';
 
@@ -682,15 +674,19 @@ void dasher_editor_action_save_state(DasherEditor *pSelf, EditorAction *pAction)
 
 void dasher_editor_output(DasherEditor *pSelf, const gchar *szText) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
-  idasher_buffer_set_insert(pPrivate->pBufferSet, szText);
+
+  if(pPrivate->pBufferSet)
+    idasher_buffer_set_insert(pPrivate->pBufferSet, szText);
 
   if(pPrivate->pGameModeHelper)
     game_mode_helper_output(pPrivate->pGameModeHelper, szText);
 }
 
 void dasher_editor_delete(DasherEditor *pSelf, int iLength) {
-  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
-  idasher_buffer_set_delete(pPrivate->pBufferSet, iLength);
+  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data); 
+
+  if(pPrivate->pBufferSet)
+    idasher_buffer_set_delete(pPrivate->pBufferSet, iLength);
 
   if(pPrivate->pGameModeHelper)
     game_mode_helper_delete(pPrivate->pGameModeHelper, iLength);
@@ -698,12 +694,16 @@ void dasher_editor_delete(DasherEditor *pSelf, int iLength) {
 
 void dasher_editor_convert(DasherEditor *pSelf) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
-  idasher_buffer_set_edit_convert(pPrivate->pBufferSet);
+
+  if(pPrivate->pBufferSet)
+    idasher_buffer_set_edit_convert(pPrivate->pBufferSet);
 }
 
 void dasher_editor_protect(DasherEditor *pSelf) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
-  idasher_buffer_set_edit_protect(pPrivate->pBufferSet);
+
+  if(pPrivate->pBufferSet)
+    idasher_buffer_set_edit_protect(pPrivate->pBufferSet);
 }
 
 EditorAction *dasher_editor_get_action_by_id(DasherEditor *pSelf, int iID){
@@ -725,10 +725,8 @@ EditorAction *dasher_editor_get_action_by_id(DasherEditor *pSelf, int iID){
 void dasher_editor_create_buffer(DasherEditor *pSelf) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
 
-#ifdef GNOME_A11Y
   if(!(pPrivate->pExternalBuffer))
     pPrivate->pExternalBuffer = IDASHER_BUFFER_SET(dasher_external_buffer_new());
-#endif
 
   if(!(pPrivate->pInternalBuffer)) {
     pPrivate->pInternalBuffer = dasher_editor_get_buffer_set(pSelf);
@@ -749,7 +747,12 @@ void dasher_editor_create_buffer(DasherEditor *pSelf) {
 void dasher_editor_refresh_context(DasherEditor *pSelf, int iMaxLength) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
 
-  gchar *szContext = idasher_buffer_set_get_context(pPrivate->pBufferSet, iMaxLength);
+  gchar *szContext;
+
+  if(pPrivate->pBufferSet)
+    szContext = idasher_buffer_set_get_context(pPrivate->pBufferSet, iMaxLength);
+  else
+    szContext = "";
   
   if(szContext && (strlen(szContext) > 0))
     gtk_dasher_control_set_context( GTK_DASHER_CONTROL(pDasherWidget), szContext );
@@ -819,6 +822,38 @@ void dasher_editor_start_tutorial(DasherEditor *pSelf) {
   pPrivate->pGameModeHelper = GAME_MODE_HELPER(game_mode_helper_new(GTK_DASHER_CONTROL(pDasherWidget)));
 }
 
+void dasher_editor_command(DasherEditor *pSelf, const gchar *szCommand) {
+  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
+
+  if(!strcmp(szCommand, "Preferences")) {
+    dasher_preferences_dialogue_show(g_pPreferencesDialogue);
+  }
+  else if(!strcmp(szCommand, "Exit")) {
+    ask_save_before_exit(NULL, NULL);
+  }
+  else if(!strcmp(szCommand, "Help")) {
+    show_help(NULL, NULL);
+  }
+  else if(!strcmp(szCommand, "About")) {
+    about_dasher(NULL, NULL);
+  }
+  else {
+    bool bActionIterStarted = false;
+    EditorAction *pActionIter = pPrivate->pActionRing;
+    
+    while((pActionIter != pPrivate->pActionRing) || !bActionIterStarted) {
+      bActionIterStarted = true;
+
+      if(!strcmp(dasher_action_get_name(pActionIter->pAction), szCommand)) {
+	dasher_action_execute(pActionIter->pAction, dasher_editor_get_all_text(pSelf));
+	return;
+      }
+
+      pActionIter = pActionIter->pNext;
+    }
+  }
+}
+
 // Callbacks
 
 extern "C" void action_button_callback(GtkWidget *pWidget, gpointer pUserData) { 
@@ -862,6 +897,13 @@ extern "C" void on_message(GtkDasherControl *pDasherControl, gpointer pMessageIn
    dasher_editor_display_message(g_pEditor, (DasherMessageInfo *)pMessageInfo);
 }
 
+
+extern "C" void on_command(GtkDasherControl *pDasherControl, gchar *szCommand, gpointer pUserData) {
+ if(g_pEditor)
+   dasher_editor_command(g_pEditor, szCommand);
+}
+
+
 // TODO: The following two should probably be made the same
 extern "C" void handle_request_settings(GtkDasherControl * pDasherControl, gpointer data) {
   dasher_preferences_dialogue_show(g_pPreferencesDialogue);
@@ -890,5 +932,5 @@ extern "C" void protect_cb(GtkDasherControl *pDasherControl, gpointer pUserData)
 
 // TODO: This should call back into editor, not directly into Dasher control
 extern "C" void context_changed_handler(GObject *pSource, gpointer pUserData) {
-  gtk_dasher_control_invalidate_context(GTK_DASHER_CONTROL(pDasherWidget), false);
+    gtk_dasher_control_invalidate_context(GTK_DASHER_CONTROL(pDasherWidget), false);
 }
