@@ -40,6 +40,8 @@ struct _DasherEditorClass {
 DasherEditor *dasher_editor_new(int argc, char **argv);
 GType dasher_editor_get_type();
 
+void dasher_editor_initialise(DasherEditor *pSelf);
+
 // TODO: Buffer set should really be private
 IDasherBufferSet *dasher_editor_get_buffer_set(DasherEditor *pSelf);
 
@@ -65,6 +67,7 @@ void dasher_editor_generate_filename(DasherEditor *pSelf);
 void dasher_editor_open(DasherEditor *pSelf, const gchar *szFilename);
 bool dasher_editor_save_as(DasherEditor *pSelf, const gchar *szFilename, bool bAppend);
 void dasher_editor_start_tutorial(DasherEditor *pSelf);
+void dasher_editor_command(DasherEditor *pSelf, const gchar *szCommand);
 
 // Temporarily here - move back to private eventually
 const gchar *dasher_editor_get_all_text(DasherEditor *pSelf);
